@@ -7,5 +7,6 @@ var 2Dfold_8h =
     [ "get_TwoDfold_variables", "group__kl__neighborhood__mfe.html#gac9284f132cf0eaa0a2f43590eda05488", null ],
     [ "destroy_TwoDfold_variables", "group__kl__neighborhood__mfe.html#ga05bf4f31d216b1b160fd2d3d68e9b487", null ],
     [ "TwoDfoldList", "group__kl__neighborhood__mfe.html#ga7fc5e3e92fe97914ca4eccd33c01c2a7", null ],
-    [ "TwoDfold_backtrack_f5", "group__kl__neighborhood__mfe.html#gaf4dc05bf8fc1ea53acd7aeb798ba80c2", null ]
+    [ "TwoDfold_backtrack_f5", "group__kl__neighborhood__mfe.html#gaf4dc05bf8fc1ea53acd7aeb798ba80c2", null ],
+    [ "TwoDfold", "group__kl__neighborhood__mfe.html#ga9827e1deba983a95d0509d36b61fb0e2", null ]
 ];
\ No newline at end of file
