@@ -1,18 +1,18 @@
 var NAVTREEINDEX7 =
 {
-"group__model__details.html#gae984567db36c3f9b8731ecc917abf3a2":[15,1,0,33],
+"group__model__details.html#gae984567db36c3f9b8731ecc917abf3a2":[17,0,0,57,31],
 "group__model__details.html#gaf1b5db10f1f476767f9a95f8a78e3132":[15,1,0,59],
 "group__model__details.html#gaf1b5db10f1f476767f9a95f8a78e3132":[17,0,0,57,57],
 "group__model__details.html#gaf47f9850b3b4763479f7a7e7a15648a2":[17,0,0,57,0],
 "group__model__details.html#gaf47f9850b3b4763479f7a7e7a15648a2":[15,1,0,2],
-"group__model__details.html#gaf9202a1a09f5828dc731e2d9a10fa111":[17,0,0,57,83],
 "group__model__details.html#gaf9202a1a09f5828dc731e2d9a10fa111":[15,1,0,85],
+"group__model__details.html#gaf9202a1a09f5828dc731e2d9a10fa111":[17,0,0,57,83],
 "group__model__details.html#gaf9e527e9a2f7e6fd6e42bc6e602f5445":[15,1,0,31],
 "group__model__details.html#gaf9e527e9a2f7e6fd6e42bc6e602f5445":[17,0,0,57,29],
 "group__model__details.html#gafb1ef1166da85092ae8a325e02dcae71":[17,0,0,57,86],
 "group__model__details.html#gafb1ef1166da85092ae8a325e02dcae71":[15,1,0,88],
-"group__model__details.html#gafff6449a02744add0308e653230c15fc":[17,0,0,57,35],
 "group__model__details.html#gafff6449a02744add0308e653230c15fc":[15,1,0,37],
+"group__model__details.html#gafff6449a02744add0308e653230c15fc":[17,0,0,57,35],
 "group__model__details.html#structvrna__md__s":[15,1,0,1],
 "group__neighbors.html":[15,2,0],
 "group__neighbors.html#a181681bc3aab907d93e340df4777e759":[15,2,0,2,2],
@@ -36,20 +36,20 @@ var NAVTREEINDEX7 =
 "group__neighbors.html#ga68ea27c81de4b74e48b775c04052590b":[15,2,0,5],
 "group__neighbors.html#ga7aa139b99eec837e646973ffa23b6a9d":[15,2,0,19],
 "group__neighbors.html#ga7aa139b99eec837e646973ffa23b6a9d":[17,0,0,3,1,12],
-"group__neighbors.html#ga9343dd62daad15b6b8c55c1260c09228":[15,2,0,10],
 "group__neighbors.html#ga9343dd62daad15b6b8c55c1260c09228":[17,0,0,3,2,2],
+"group__neighbors.html#ga9343dd62daad15b6b8c55c1260c09228":[15,2,0,10],
 "group__neighbors.html#ga9cf75e9c6551f46ea568abf4f7fd11cd":[15,2,0,16],
 "group__neighbors.html#ga9cf75e9c6551f46ea568abf4f7fd11cd":[17,0,0,3,1,9],
 "group__neighbors.html#gaa3e8c627d24912974922c5ae805e15e0":[15,2,0,24],
 "group__neighbors.html#gaa3e8c627d24912974922c5ae805e15e0":[17,0,0,3,2,8],
-"group__neighbors.html#gaa5ffec4dd0d02df320f123e6888154d1":[15,2,0,7],
 "group__neighbors.html#gaa5ffec4dd0d02df320f123e6888154d1":[17,0,0,3,1,4],
-"group__neighbors.html#gac05db9392c6647e3e9a6982096c5b384":[15,2,0,4],
+"group__neighbors.html#gaa5ffec4dd0d02df320f123e6888154d1":[15,2,0,7],
 "group__neighbors.html#gac05db9392c6647e3e9a6982096c5b384":[17,0,0,3,1,1],
-"group__neighbors.html#gacdbc5f609b46aeb07d2c7e015677a1e0":[15,2,0,15],
+"group__neighbors.html#gac05db9392c6647e3e9a6982096c5b384":[15,2,0,4],
 "group__neighbors.html#gacdbc5f609b46aeb07d2c7e015677a1e0":[17,0,0,3,1,8],
-"group__neighbors.html#gad085062ae3aa7ee20783debe4a906fad":[15,2,0,14],
+"group__neighbors.html#gacdbc5f609b46aeb07d2c7e015677a1e0":[15,2,0,15],
 "group__neighbors.html#gad085062ae3aa7ee20783debe4a906fad":[17,0,0,3,1,7],
+"group__neighbors.html#gad085062ae3aa7ee20783debe4a906fad":[15,2,0,14],
 "group__neighbors.html#gad495362605ffd0168591ce41a685ee3e":[15,2,0,23],
 "group__neighbors.html#gad495362605ffd0168591ce41a685ee3e":[17,0,0,3,2,7],
 "group__neighbors.html#gae5aaa1c5a1f22e889843f3edbdd04714":[15,2,0,22],
@@ -58,8 +58,8 @@ var NAVTREEINDEX7 =
 "group__neighbors.html#gae72b0e349b6108c00737df51cce44622":[17,0,0,3,2,0],
 "group__neighbors.html#gaeff6157398a2073f587416fa9ec94789":[17,0,0,3,1,6],
 "group__neighbors.html#gaeff6157398a2073f587416fa9ec94789":[15,2,0,13],
-"group__neighbors.html#gaf39028db9c70d3be528929182a3f2d5a":[17,0,0,3,1,0],
 "group__neighbors.html#gaf39028db9c70d3be528929182a3f2d5a":[15,2,0,3],
+"group__neighbors.html#gaf39028db9c70d3be528929182a3f2d5a":[17,0,0,3,1,0],
 "group__neighbors.html#gaf7e3417828057ae0ed4d0de03178116a":[17,0,0,3,2,1],
 "group__neighbors.html#gaf7e3417828057ae0ed4d0de03178116a":[15,2,0,9],
 "group__neighbors.html#structvrna__move__s":[15,2,0,2],
@@ -94,8 +94,8 @@ var NAVTREEINDEX7 =
 "group__part__func__global.html#gaaf197722d1faa86af5e7b4240acafdee":[15,4,0,6],
 "group__part__func__global.html#gab70fe6c9a78b79cc5669881720926e1d":[17,0,0,64,7],
 "group__part__func__global.html#gab70fe6c9a78b79cc5669881720926e1d":[15,4,0,15],
-"group__part__func__global.html#gac4a2a74a79e49818bc35412a2b392c7e":[17,0,0,64,4],
 "group__part__func__global.html#gac4a2a74a79e49818bc35412a2b392c7e":[15,4,0,12],
+"group__part__func__global.html#gac4a2a74a79e49818bc35412a2b392c7e":[17,0,0,64,4],
 "group__part__func__global.html#gad3f0c240512e6d43e2e4d4c2076021f5":[17,0,0,35,0],
 "group__part__func__global.html#gad3f0c240512e6d43e2e4d4c2076021f5":[15,4,0,4],
 "group__part__func__global.html#gaf2b846f7ac382686f35ff7b9202fdd5c":[17,0,0,64,8],
@@ -111,14 +111,14 @@ var NAVTREEINDEX7 =
 "group__part__func__global__deprecated.html#ga11b6ab8bd9be1821fea352b190a01cab":[15,4,0,1,27],
 "group__part__func__global__deprecated.html#ga15176e23eceeff8c7d14eabcfec8a2af":[17,0,0,64,25],
 "group__part__func__global__deprecated.html#ga15176e23eceeff8c7d14eabcfec8a2af":[15,4,0,1,14],
-"group__part__func__global__deprecated.html#ga189e2a1ec6cc32c53ea72f7543b0441e":[17,0,0,64,21],
 "group__part__func__global__deprecated.html#ga189e2a1ec6cc32c53ea72f7543b0441e":[15,4,0,1,10],
+"group__part__func__global__deprecated.html#ga189e2a1ec6cc32c53ea72f7543b0441e":[17,0,0,64,21],
 "group__part__func__global__deprecated.html#ga1a5f6cfb9d761fa862ce4edc7c369cd2":[17,0,0,12,5],
 "group__part__func__global__deprecated.html#ga1a5f6cfb9d761fa862ce4edc7c369cd2":[15,4,0,1,25],
 "group__part__func__global__deprecated.html#ga1cc05aaa9b0e7df2d3887e98321c2030":[17,0,0,8,5,61],
 "group__part__func__global__deprecated.html#ga1cc05aaa9b0e7df2d3887e98321c2030":[15,4,0,1,24],
-"group__part__func__global__deprecated.html#ga384e927890f9c034ff09fa66da102d28":[15,4,0,1,6],
 "group__part__func__global__deprecated.html#ga384e927890f9c034ff09fa66da102d28":[17,0,0,64,17],
+"group__part__func__global__deprecated.html#ga384e927890f9c034ff09fa66da102d28":[15,4,0,1,6],
 "group__part__func__global__deprecated.html#ga42faebdfce6f070c5f89adfc8427525c":[17,0,0,64,20],
 "group__part__func__global__deprecated.html#ga42faebdfce6f070c5f89adfc8427525c":[15,4,0,1,9],
 "group__part__func__global__deprecated.html#ga5349960075b1847720a2e9df021e2675":[17,0,0,12,10],
@@ -133,12 +133,12 @@ var NAVTREEINDEX7 =
 "group__part__func__global__deprecated.html#ga79cbc375af65f11609feb6b055269e7d":[15,4,0,1,11],
 "group__part__func__global__deprecated.html#ga819ce5fca8984004ac81c4a3b04cb735":[17,0,0,64,12],
 "group__part__func__global__deprecated.html#ga819ce5fca8984004ac81c4a3b04cb735":[15,4,0,1,4],
-"group__part__func__global__deprecated.html#ga8f18624bcd0b47d1a266d9e5527e5362":[15,4,0,1,13],
 "group__part__func__global__deprecated.html#ga8f18624bcd0b47d1a266d9e5527e5362":[17,0,0,64,24],
-"group__part__func__global__deprecated.html#ga94c19120130e66a667a10a3c8598550c":[17,0,0,65,3],
+"group__part__func__global__deprecated.html#ga8f18624bcd0b47d1a266d9e5527e5362":[15,4,0,1,13],
 "group__part__func__global__deprecated.html#ga94c19120130e66a667a10a3c8598550c":[15,4,0,1,17],
-"group__part__func__global__deprecated.html#gaa12dda9dd6179cdd22bcce87c0682c07":[15,4,0,1,18],
+"group__part__func__global__deprecated.html#ga94c19120130e66a667a10a3c8598550c":[17,0,0,65,3],
 "group__part__func__global__deprecated.html#gaa12dda9dd6179cdd22bcce87c0682c07":[17,0,0,65,4],
+"group__part__func__global__deprecated.html#gaa12dda9dd6179cdd22bcce87c0682c07":[15,4,0,1,18],
 "group__part__func__global__deprecated.html#gaabfc6cb6d02b8f08ac4c92f4f5b125d9":[15,4,0,1,16],
 "group__part__func__global__deprecated.html#gaabfc6cb6d02b8f08ac4c92f4f5b125d9":[17,0,0,65,1],
 "group__part__func__global__deprecated.html#gaafe2d1b21f5418b123b088aa395e827d":[17,0,0,64,18],
@@ -151,14 +151,14 @@ var NAVTREEINDEX7 =
 "group__part__func__global__deprecated.html#gac4f95bee734b2563a3d6e9932117ebdf":[15,4,0,1,2],
 "group__part__func__global__deprecated.html#gac5ac7ee281aae1c5cc5898a841178073":[17,0,0,64,19],
 "group__part__func__global__deprecated.html#gac5ac7ee281aae1c5cc5898a841178073":[15,4,0,1,8],
-"group__part__func__global__deprecated.html#gad5ba36cef8d01cf4244cc09b9bf1ce1d":[15,4,0,1,12],
 "group__part__func__global__deprecated.html#gad5ba36cef8d01cf4244cc09b9bf1ce1d":[17,0,0,64,23],
+"group__part__func__global__deprecated.html#gad5ba36cef8d01cf4244cc09b9bf1ce1d":[15,4,0,1,12],
 "group__part__func__global__deprecated.html#gad94c0133157bed6912fe7fe866e0039e":[15,4,0,1,19],
 "group__part__func__global__deprecated.html#gad94c0133157bed6912fe7fe866e0039e":[17,0,0,65,5],
 "group__part__func__global__deprecated.html#gadc3db3d98742427e7001a7fd36ef28c2":[17,0,0,64,11],
 "group__part__func__global__deprecated.html#gadc3db3d98742427e7001a7fd36ef28c2":[15,4,0,1,3],
-"group__part__func__global__deprecated.html#gade3ce34ae8214811374b1d28a40dc247":[17,0,0,65,6],
 "group__part__func__global__deprecated.html#gade3ce34ae8214811374b1d28a40dc247":[15,4,0,1,20],
+"group__part__func__global__deprecated.html#gade3ce34ae8214811374b1d28a40dc247":[17,0,0,65,6],
 "group__part__func__global__deprecated.html#gae5c1e7331718669bdae7a86de2be6184":[15,4,0,1,15],
 "group__part__func__global__deprecated.html#gae5c1e7331718669bdae7a86de2be6184":[17,0,0,65,0],
 "group__part__func__global__deprecated.html#gae73db3f49a94f0f72e067ecd12681dbd":[15,4,0,1,5],
@@ -168,10 +168,10 @@ var NAVTREEINDEX7 =
 "group__part__func__window.html#ga0a13fc668b02f21b225e2334e4caac08":[15,4,1,9],
 "group__part__func__window.html#ga18325811c7dfc7b7d9d4ac37f4353615":[17,0,0,67,6],
 "group__part__func__window.html#ga18325811c7dfc7b7d9d4ac37f4353615":[15,4,1,8],
-"group__part__func__window.html#ga1dd5c51b797c961124912e289bff553a":[15,4,1,16],
 "group__part__func__window.html#ga1dd5c51b797c961124912e289bff553a":[17,0,0,67,14],
-"group__part__func__window.html#ga1fef1ad234755db78ab6b695edce5080":[17,0,0,67,4],
+"group__part__func__window.html#ga1dd5c51b797c961124912e289bff553a":[15,4,1,16],
 "group__part__func__window.html#ga1fef1ad234755db78ab6b695edce5080":[15,4,1,6],
+"group__part__func__window.html#ga1fef1ad234755db78ab6b695edce5080":[17,0,0,67,4],
 "group__part__func__window.html#ga296217b76e76e5f7e6927e7210aa9b1f":[17,0,0,67,5],
 "group__part__func__window.html#ga296217b76e76e5f7e6927e7210aa9b1f":[15,4,1,7],
 "group__part__func__window.html#ga30170811c320f3a7b23ffff3b3343b91":[17,0,0,67,1],
@@ -180,20 +180,20 @@ var NAVTREEINDEX7 =
 "group__part__func__window.html#ga457a60751b2a5225477e3b7735636a5f":[15,4,1,15],
 "group__part__func__window.html#ga4b79137d9b28b1f9cac7983792ce34a0":[17,0,0,67,9],
 "group__part__func__window.html#ga4b79137d9b28b1f9cac7983792ce34a0":[15,4,1,11],
-"group__part__func__window.html#ga6267230f20cab0e2315375310b4dad85":[17,0,0,67,12],
 "group__part__func__window.html#ga6267230f20cab0e2315375310b4dad85":[15,4,1,14],
-"group__part__func__window.html#ga7115d012988541a65ec323c5f17a334b":[17,0,0,67,11],
+"group__part__func__window.html#ga6267230f20cab0e2315375310b4dad85":[17,0,0,67,12],
 "group__part__func__window.html#ga7115d012988541a65ec323c5f17a334b":[15,4,1,13],
+"group__part__func__window.html#ga7115d012988541a65ec323c5f17a334b":[17,0,0,67,11],
 "group__part__func__window.html#ga9068f4ec008bf1c042a9357f5c2c5e13":[17,0,0,67,8],
 "group__part__func__window.html#ga9068f4ec008bf1c042a9357f5c2c5e13":[15,4,1,10],
 "group__part__func__window.html#ga956a237e7d956b684b7d593cb3eca665":[17,0,0,67,2],
 "group__part__func__window.html#ga956a237e7d956b684b7d593cb3eca665":[15,4,1,4],
 "group__part__func__window.html#gabe710a1182e6db69cc75329dfc9bed67":[17,0,0,67,10],
 "group__part__func__window.html#gabe710a1182e6db69cc75329dfc9bed67":[15,4,1,12],
-"group__part__func__window.html#gac3251d3da0238d6d9ffdd6703b00f1d3":[15,4,1,17],
 "group__part__func__window.html#gac3251d3da0238d6d9ffdd6703b00f1d3":[17,0,0,67,15],
-"group__part__func__window.html#gad905b71f02b3799eb84b490435aec837":[17,0,0,67,3],
+"group__part__func__window.html#gac3251d3da0238d6d9ffdd6703b00f1d3":[15,4,1,17],
 "group__part__func__window.html#gad905b71f02b3799eb84b490435aec837":[15,4,1,5],
+"group__part__func__window.html#gad905b71f02b3799eb84b490435aec837":[17,0,0,67,3],
 "group__part__func__window.html#gaf0bb577130090f6c856d1358804250bf":[17,0,0,67,0],
 "group__part__func__window.html#gaf0bb577130090f6c856d1358804250bf":[15,4,1,2],
 "group__part__func__window__deprecated.html":[15,4,1,0],
@@ -203,8 +203,8 @@ var NAVTREEINDEX7 =
 "group__part__func__window__deprecated.html#ga5a019014d37fe6105131dfc2fc447880":[17,0,0,52,0],
 "group__part__func__window__deprecated.html#ga7cdf690583871f9c158a0f91d4785cb6":[17,0,0,52,1],
 "group__part__func__window__deprecated.html#ga7cdf690583871f9c158a0f91d4785cb6":[15,4,1,0,2],
-"group__part__func__window__deprecated.html#ga9acb00ee10e96b1ca4ea394cd8bcec75":[15,4,1,0,5],
 "group__part__func__window__deprecated.html#ga9acb00ee10e96b1ca4ea394cd8bcec75":[17,0,0,52,4],
+"group__part__func__window__deprecated.html#ga9acb00ee10e96b1ca4ea394cd8bcec75":[15,4,1,0,5],
 "group__part__func__window__deprecated.html#gaedf355f07b3d83bc93f98712132571b0":[17,0,0,52,2],
 "group__part__func__window__deprecated.html#gaedf355f07b3d83bc93f98712132571b0":[15,4,1,0,3],
 "group__paths.html":[15,2,1],
@@ -212,8 +212,8 @@ var NAVTREEINDEX7 =
 "group__paths.html#a141b70a59cb81d10bc65bbb7a0f6db77":[15,2,1,6,2],
 "group__paths.html#a5fc9e0a7dc0e0f8ad67f24c18f2dc852":[15,2,1,6,0],
 "group__paths.html#ac25160bf31d28097358278f367e41227":[15,2,1,6,1],
-"group__paths.html#ga3763dc16760d09277a1b1c3f3061d0c7":[15,2,1,11],
 "group__paths.html#ga3763dc16760d09277a1b1c3f3061d0c7":[17,0,0,3,3,5],
+"group__paths.html#ga3763dc16760d09277a1b1c3f3061d0c7":[15,2,1,11],
 "group__paths.html#ga3e964156b52440b65fae5e2a48750d8e":[15,2,1,7],
 "group__paths.html#ga3e964156b52440b65fae5e2a48750d8e":[17,0,0,3,3,0],
 "group__paths.html#ga818d4f3d1cf8723d6905990b08d909fe":[17,0,0,3,3,2],
@@ -226,12 +226,12 @@ var NAVTREEINDEX7 =
 "group__paths.html#gaa99d8ec48f64de53eda0d51b1ed0d3d1":[15,2,1,10],
 "group__paths.html#structvrna__path__s":[15,2,1,6],
 "group__paths__deprecated.html":[15,2,1,2],
-"group__paths__deprecated.html#ga4c0dabf032c04eeab9c7370d15db6ad2":[17,0,0,3,0,4],
 "group__paths__deprecated.html#ga4c0dabf032c04eeab9c7370d15db6ad2":[15,2,1,2,1],
-"group__paths__deprecated.html#ga9056421d716ae89f0ed3f107627f395b":[15,2,1,2,2],
+"group__paths__deprecated.html#ga4c0dabf032c04eeab9c7370d15db6ad2":[17,0,0,3,0,4],
 "group__paths__deprecated.html#ga9056421d716ae89f0ed3f107627f395b":[17,0,0,3,0,5],
-"group__paths__deprecated.html#gab6b8737d5377e70a7815d04aae7fd884":[15,2,1,2,0],
+"group__paths__deprecated.html#ga9056421d716ae89f0ed3f107627f395b":[15,2,1,2,2],
 "group__paths__deprecated.html#gab6b8737d5377e70a7815d04aae7fd884":[17,0,0,3,3,4],
+"group__paths__deprecated.html#gab6b8737d5377e70a7815d04aae7fd884":[15,2,1,2,0],
 "group__paths__deprecated.html#gac009a44db824f90a6de5f9c28b9b3222":[15,2,1,2,3],
 "group__paths__deprecated.html#gac009a44db824f90a6de5f9c28b9b3222":[17,0,0,3,0,6],
 "group__paths__direct.html":[15,2,1,0],
@@ -239,15 +239,15 @@ var NAVTREEINDEX7 =
 "group__paths__direct.html#ga2fed8ff126f5becd82cedd81a25c9bf0":[17,0,0,3,0,2],
 "group__paths__direct.html#ga3790a24ca64d68acfa6e67b525151a5f":[15,2,1,0,4],
 "group__paths__direct.html#ga3790a24ca64d68acfa6e67b525151a5f":[17,0,0,3,3,7],
-"group__paths__direct.html#ga51d9edaeb42d1d178ea8ba46ccd4c8c0":[17,0,0,3,3,9],
 "group__paths__direct.html#ga51d9edaeb42d1d178ea8ba46ccd4c8c0":[15,2,1,0,6],
-"group__paths__direct.html#gaca6d3ab6f8b9463e3382bd1f474c26f1":[17,0,0,3,0,1],
+"group__paths__direct.html#ga51d9edaeb42d1d178ea8ba46ccd4c8c0":[17,0,0,3,3,9],
 "group__paths__direct.html#gaca6d3ab6f8b9463e3382bd1f474c26f1":[15,2,1,0,1],
+"group__paths__direct.html#gaca6d3ab6f8b9463e3382bd1f474c26f1":[17,0,0,3,0,1],
 "group__paths__direct.html#gacba096caf9a6fe134f33a5afcf2248d8":[17,0,0,3,3,8],
 "group__paths__direct.html#gacba096caf9a6fe134f33a5afcf2248d8":[15,2,1,0,5],
 "group__paths__direct.html#gad00765c2f269fddfa319310af22d4f67":[17,0,0,3,0,3],
 "group__paths__direct.html#gad00765c2f269fddfa319310af22d4f67":[15,2,1,0,3],
-"group__paths__direct.html#gad77dbfa125158f534147c0015a849bc3":[15,2,1,0,0],
 "group__paths__direct.html#gad77dbfa125158f534147c0015a849bc3":[17,0,0,3,0,0],
+"group__paths__direct.html#gad77dbfa125158f534147c0015a849bc3":[15,2,1,0,0],
 "group__paths__walk.html":[15,2,1,1]
 };
