@@ -1,8 +1,8 @@
-/* A Bison parser, made by GNU Bison 3.0.4.  */
+/* A Bison parser, made by GNU Bison 3.0.5.  */
 
 /* Bison implementation for Yacc-like parsers in C
 
-   Copyright (C) 1984, 1989-1990, 2000-2015 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989-1990, 2000-2015, 2018 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@
 #define YYBISON 1
 
 /* Bison version.  */
-#define YYBISON_VERSION "3.0.4"
+#define YYBISON_VERSION "3.0.5"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -974,6 +974,7 @@ yysyntax_error (YYSIZE_T *yymsg_alloc, char **yymsg,
       case N:                               \
         yyformat = S;                       \
       break
+    default: /* Avoid compiler warnings. */
       YYCASE_(0, YY_("syntax error"));
       YYCASE_(1, YY_("syntax error, unexpected %s"));
       YYCASE_(2, YY_("syntax error, unexpected %s, expecting %s"));
@@ -1309,13 +1310,14 @@ yyreduce:
      GCC warning that YYVAL may be used uninitialized.  */
   yyval = yyvsp[1-yylen];
 
-  /* Default location.  */
+  /* Default location. */
   YYLLOC_DEFAULT (yyloc, (yylsp - yylen), yylen);
+  yyerror_range[1] = yyloc;
   YY_REDUCE_PRINT (yyn);
   switch (yyn)
     {
         case 2:
-#line 33 "anchors/shape.ypp" /* yacc.c:1646  */
+#line 33 "anchors/shape.ypp" /* yacc.c:1648  */
     {
 
      for (std::list<Pair*>::iterator i = (yyvsp[0].pairs)->begin(); i != (yyvsp[0].pairs)->end(); ++i)
@@ -1323,30 +1325,30 @@ yyreduce:
      for (std::list<Pair*>::iterator i = (yyvsp[0].pairs)->begin(); i != (yyvsp[0].pairs)->end(); ++i)
        (*i)->put(anchorlist, std::cout);
 }
-#line 1327 "anchors/shape.cpp" /* yacc.c:1646  */
+#line 1329 "anchors/shape.cpp" /* yacc.c:1648  */
     break;
 
   case 3:
-#line 41 "anchors/shape.ypp" /* yacc.c:1646  */
+#line 41 "anchors/shape.ypp" /* yacc.c:1648  */
     { (yyvsp[-1].pairs)->push_back((yyvsp[0].pair)); (yyval.pairs) = (yyvsp[-1].pairs); }
-#line 1333 "anchors/shape.cpp" /* yacc.c:1646  */
+#line 1335 "anchors/shape.cpp" /* yacc.c:1648  */
     break;
 
   case 4:
-#line 42 "anchors/shape.ypp" /* yacc.c:1646  */
+#line 42 "anchors/shape.ypp" /* yacc.c:1648  */
     { (yyval.pairs) = new std::list<Pair*>(); }
-#line 1339 "anchors/shape.cpp" /* yacc.c:1646  */
+#line 1341 "anchors/shape.cpp" /* yacc.c:1648  */
     break;
 
   case 5:
-#line 44 "anchors/shape.ypp" /* yacc.c:1646  */
+#line 44 "anchors/shape.ypp" /* yacc.c:1648  */
     { 
      (yyval.pair) = new Pair((yyvsp[-1].pairs),(yylsp[-1]).first_column); }
-#line 1346 "anchors/shape.cpp" /* yacc.c:1646  */
+#line 1348 "anchors/shape.cpp" /* yacc.c:1648  */
     break;
 
 
-#line 1350 "anchors/shape.cpp" /* yacc.c:1646  */
+#line 1352 "anchors/shape.cpp" /* yacc.c:1648  */
       default: break;
     }
   /* User semantic actions sometimes alter yychar, and that requires
@@ -1469,7 +1471,6 @@ yyerrorlab:
   if (/*CONSTCOND*/ 0)
      goto yyerrorlab;
 
-  yyerror_range[1] = yylsp[1-yylen];
   /* Do not reclaim the symbols of the rule whose action triggered
      this YYERROR.  */
   YYPOPSTACK (yylen);
@@ -1581,7 +1582,7 @@ yyreturn:
 #endif
   return yyresult;
 }
-#line 47 "anchors/shape.ypp" /* yacc.c:1906  */
+#line 47 "anchors/shape.ypp" /* yacc.c:1907  */
 
 
 extern
