@@ -99,9 +99,11 @@ typedef void (vrna_callback_recursion_status)(unsigned char status,
 
 
 #include <ViennaRNA/model.h>
-#include <ViennaRNA/params.h>
+#include <ViennaRNA/params/basic.h>
+#include <ViennaRNA/sequence.h>
 #include <ViennaRNA/dp_matrices.h>
-#include <ViennaRNA/constraints.h>
+#include <ViennaRNA/constraints/hard.h>
+#include <ViennaRNA/constraints/soft.h>
 #include <ViennaRNA/grammar.h>
 #include "ViennaRNA/structured_domains.h"
 #include "ViennaRNA/unstructured_domains.h"
@@ -132,7 +134,7 @@ struct vrna_fc_s {
    *  @name Common data fields
    *  @{
    */
-  vrna_fc_type_e type;              /**<  @brief  The type of the #vrna_fold_compound_t.
+  const vrna_fc_type_e type;        /**<  @brief  The type of the #vrna_fold_compound_t.
                                      * @details Currently possible values are #VRNA_FC_TYPE_SINGLE, and #VRNA_FC_TYPE_COMPARATIVE
                                      * @warning Do not edit this attribute, it will be automagically set by
                                      *      the corresponding get() methods for the #vrna_fold_compound_t.
@@ -145,6 +147,13 @@ struct vrna_fc_s {
                                      */
 
   unsigned int      *strand_number; /**<  @brief  The strand number a particular nucleotide is associated with */
+  unsigned int      *strand_order;  /**<  @brief  The strand order, i.e. permutation of current concatenated sequence */
+  unsigned int      *strand_start;  /**<  @brief  The start position of a particular strand within the current concatenated sequence */
+  unsigned int      *strand_end;    /**<  @brief  The end (last) position of a particular strand within the current concatenated sequence */
+
+  unsigned int      strands;
+  vrna_seq_t        *nucleotides;
+  vrna_msa_t        *alignment;
 
   vrna_hc_t         *hc;            /**<  @brief  The hard constraints data structure used for structure prediction */
 
@@ -206,31 +215,31 @@ struct vrna_fc_s {
    *  @name Data fields available for single/hybrid structure prediction
    *  @{
    */
-  char *sequence;                   /**<  @brief  The input sequence string
-                                     *    @warning   Only available if @verbatim type==VRNA_FC_TYPE_SINGLE @endverbatim
-                                     */
-  short *sequence_encoding;         /**<  @brief  Numerical encoding of the input sequence
-                                     *    @see    vrna_sequence_encode()
-                                     *    @warning   Only available if @verbatim type==VRNA_FC_TYPE_SINGLE @endverbatim
-                                     */
+      char *sequence;                   /**<  @brief  The input sequence string
+                                         *    @warning   Only available if @verbatim type==VRNA_FC_TYPE_SINGLE @endverbatim
+                                         */
+      short *sequence_encoding;         /**<  @brief  Numerical encoding of the input sequence
+                                         *    @see    vrna_sequence_encode()
+                                         *    @warning   Only available if @verbatim type==VRNA_FC_TYPE_SINGLE @endverbatim
+                                         */
   short *sequence_encoding2;
-  char *ptype;                      /**<  @brief  Pair type array
-                                     *
-                                     *    Contains the numerical encoding of the pair type for each pair (i,j) used
-                                     *    in MFE, Partition function and Evaluation computations.
-                                     *    @note This array is always indexed via jindx, in contrast to previously
-                                     *    different indexing between mfe and pf variants!
-                                     *    @warning   Only available if @verbatim type==VRNA_FC_TYPE_SINGLE @endverbatim
-                                     *    @see    vrna_idx_col_wise(), vrna_ptypes()
-                                     */
-  char *ptype_pf_compat;            /**<  @brief  ptype array indexed via iindx
-                                     *    @deprecated  This attribute will vanish in the future!
-                                     *    It's meant for backward compatibility only!
-                                     *    @warning   Only available if @verbatim type==VRNA_FC_TYPE_SINGLE @endverbatim
-                                     */
-  vrna_sc_t *sc;                    /**<  @brief  The soft constraints for usage in structure prediction and evaluation
-                                     *    @warning   Only available if @verbatim type==VRNA_FC_TYPE_SINGLE @endverbatim
-                                     */
+      char *ptype;                      /**<  @brief  Pair type array
+                                         *
+                                         *    Contains the numerical encoding of the pair type for each pair (i,j) used
+                                         *    in MFE, Partition function and Evaluation computations.
+                                         *    @note This array is always indexed via jindx, in contrast to previously
+                                         *    different indexing between mfe and pf variants!
+                                         *    @warning   Only available if @verbatim type==VRNA_FC_TYPE_SINGLE @endverbatim
+                                         *    @see    vrna_idx_col_wise(), vrna_ptypes()
+                                         */
+      char *ptype_pf_compat;            /**<  @brief  ptype array indexed via iindx
+                                         *    @deprecated  This attribute will vanish in the future!
+                                         *    It's meant for backward compatibility only!
+                                         *    @warning   Only available if @verbatim type==VRNA_FC_TYPE_SINGLE @endverbatim
+                                         */
+      vrna_sc_t *sc;                    /**<  @brief  The soft constraints for usage in structure prediction and evaluation
+                                         *    @warning   Only available if @verbatim type==VRNA_FC_TYPE_SINGLE @endverbatim
+                                         */
 
   /**
    *  @}
@@ -246,44 +255,44 @@ struct {
    *  @name Data fields for consensus structure prediction
    *  @{
    */
-  char            **sequences;      /**<  @brief  The aligned sequences
-                                     *    @note   The end of the alignment is indicated by a NULL pointer in the second dimension
-                                     *    @warning   Only available if @verbatim type==VRNA_FC_TYPE_COMPARATIVE @endverbatim
-                                     */
-  unsigned int    n_seq;            /**<  @brief  The number of sequences in the alignment
-                                     *    @warning   Only available if @verbatim type==VRNA_FC_TYPE_COMPARATIVE @endverbatim
-                                     */
-  char            *cons_seq;        /**<  @brief  The consensus sequence of the aligned sequences
-                                     *    @warning   Only available if @verbatim type==VRNA_FC_TYPE_COMPARATIVE @endverbatim
-                                     */
-  short           *S_cons;          /**<  @brief  Numerical encoding of the consensus sequence
-                                     *    @warning   Only available if @verbatim type==VRNA_FC_TYPE_COMPARATIVE @endverbatim
-                                     */
-  short           **S;              /**<  @brief  Numerical encoding of the sequences in the alignment
-                                     *    @warning   Only available if @verbatim type==VRNA_FC_TYPE_COMPARATIVE @endverbatim
-                                     */
-  short           **S5;             /**<  @brief    S5[s][i] holds next base 5' of i in sequence s
-                                     *    @warning  Only available if @verbatim type==VRNA_FC_TYPE_COMPARATIVE @endverbatim
-                                     */
-  short           **S3;             /**<  @brief    Sl[s][i] holds next base 3' of i in sequence s
-                                     *    @warning  Only available if @verbatim type==VRNA_FC_TYPE_COMPARATIVE @endverbatim
-                                     */
-  char            **Ss;
-  unsigned int    **a2s;
-  int             *pscore;            /**<  @brief  Precomputed array of pair types expressed as pairing scores
-                                       *    @warning   Only available if @verbatim type==VRNA_FC_TYPE_COMPARATIVE @endverbatim
-                                       */
-  int             **pscore_local;     /**<  @brief  Precomputed array of pair types expressed as pairing scores
-                                       *    @warning   Only available if @verbatim type==VRNA_FC_TYPE_COMPARATIVE @endverbatim
-                                       */
-  short           *pscore_pf_compat;  /**<  @brief  Precomputed array of pair types expressed as pairing scores indexed via iindx
-                                       *    @deprecated  This attribute will vanish in the future!
-                                       *    @warning   Only available if @verbatim type==VRNA_FC_TYPE_COMPARATIVE @endverbatim
-                                       */
-  vrna_sc_t       **scs;              /**<  @brief  A set of soft constraints (for each sequence in the alignment)
-                                       *    @warning   Only available if @verbatim type==VRNA_FC_TYPE_COMPARATIVE @endverbatim
-                                       */
-  int             oldAliEn;
+      char          **sequences;        /**<  @brief  The aligned sequences
+                                         *    @note   The end of the alignment is indicated by a NULL pointer in the second dimension
+                                         *    @warning   Only available if @verbatim type==VRNA_FC_TYPE_COMPARATIVE @endverbatim
+                                         */
+      unsigned int  n_seq;              /**<  @brief  The number of sequences in the alignment
+                                         *    @warning   Only available if @verbatim type==VRNA_FC_TYPE_COMPARATIVE @endverbatim
+                                         */
+      char          *cons_seq;          /**<  @brief  The consensus sequence of the aligned sequences
+                                         *    @warning   Only available if @verbatim type==VRNA_FC_TYPE_COMPARATIVE @endverbatim
+                                         */
+      short         *S_cons;            /**<  @brief  Numerical encoding of the consensus sequence
+                                         *    @warning   Only available if @verbatim type==VRNA_FC_TYPE_COMPARATIVE @endverbatim
+                                         */
+      short         **S;                /**<  @brief  Numerical encoding of the sequences in the alignment
+                                         *    @warning   Only available if @verbatim type==VRNA_FC_TYPE_COMPARATIVE @endverbatim
+                                         */
+      short         **S5;               /**<  @brief    S5[s][i] holds next base 5' of i in sequence s
+                                         *    @warning  Only available if @verbatim type==VRNA_FC_TYPE_COMPARATIVE @endverbatim
+                                         */
+      short         **S3;               /**<  @brief    Sl[s][i] holds next base 3' of i in sequence s
+                                         *    @warning  Only available if @verbatim type==VRNA_FC_TYPE_COMPARATIVE @endverbatim
+                                         */
+  char          **Ss;
+  unsigned int  **a2s;
+      int           *pscore;              /**<  @brief  Precomputed array of pair types expressed as pairing scores
+                                           *    @warning   Only available if @verbatim type==VRNA_FC_TYPE_COMPARATIVE @endverbatim
+                                           */
+      int           **pscore_local;       /**<  @brief  Precomputed array of pair types expressed as pairing scores
+                                           *    @warning   Only available if @verbatim type==VRNA_FC_TYPE_COMPARATIVE @endverbatim
+                                           */
+      short         *pscore_pf_compat;    /**<  @brief  Precomputed array of pair types expressed as pairing scores indexed via iindx
+                                           *    @deprecated  This attribute will vanish in the future!
+                                           *    @warning   Only available if @verbatim type==VRNA_FC_TYPE_COMPARATIVE @endverbatim
+                                           */
+      vrna_sc_t     **scs;                /**<  @brief  A set of soft constraints (for each sequence in the alignment)
+                                           *    @warning   Only available if @verbatim type==VRNA_FC_TYPE_COMPARATIVE @endverbatim
+                                           */
+  int           oldAliEn;
 
   /**
    *  @}
@@ -384,15 +393,14 @@ struct {
  *  based on the content of the generated #vrna_fold_compound_t. Passing NULL will instruct the function
  *  to use default model details.
  *  The third parameter @p options may be used to specify dynamic programming (DP) matrix requirements.
- *  Use the macros:
  *
- *  - #VRNA_OPTION_MFE
- *  - #VRNA_OPTION_PF
- *  - #VRNA_OPTION_WINDOW
- *  - #VRNA_OPTION_EVAL_ONLY
- *  - #VRNA_OPTION_DEFAULT
+ *  #### Options ####
+ *  * #VRNA_OPTION_DEFAULT  - @copybrief #VRNA_OPTION_DEFAULT
+ *  * #VRNA_OPTION_MFE      - @copybrief #VRNA_OPTION_MFE
+ *  * #VRNA_OPTION_PF       - @copybrief #VRNA_OPTION_PF
+ *  * #VRNA_OPTION_WINDOW   - @copybrief #VRNA_OPTION_WINDOW
  *
- *  to specify the required type of computations that will be performed with the #vrna_fold_compound_t.
+ *  The above options may be OR-ed together.
  *
  *  If you just need the folding compound serving as a container for your data, you can simply pass
  *  #VRNA_OPTION_DEFAULT to the @p option parameter. This creates a #vrna_fold_compound_t without DP
@@ -404,13 +412,12 @@ struct {
  *  @note The sequence string must be uppercase, and should contain only RNA (resp. DNA) alphabet depending
  *        on what energy parameter set is used
  *
- *  @see  vrna_fold_compound_free(), vrna_fold_compound_comparative(), #vrna_md_t, #VRNA_OPTION_MFE,
- *        #VRNA_OPTION_PF, #VRNA_OPTION_EVAL_ONLY, #VRNA_OPTION_WINDOW
+ *  @see  vrna_fold_compound_free(), vrna_fold_compound_comparative(), #vrna_md_t
  *
  *  @param    sequence    A single sequence, or two concatenated sequences seperated by an '&' character
  *  @param    md_p        An optional set of model details
  *  @param    options     The options for DP matrices memory allocation
- *  @return               A prefilled vrna_fold_compound_t that can be readily used for computations
+ *  @return               A prefilled vrna_fold_compound_t ready to be used for computations (may be @p NULL on error)
  */
 vrna_fold_compound_t *
 vrna_fold_compound(const char   *sequence,
@@ -428,14 +435,14 @@ vrna_fold_compound(const char   *sequence,
  *  based on the content of the generated #vrna_fold_compound_t. Passing NULL will instruct the function
  *  to use default model details.
  *  The third parameter @p options may be used to specify dynamic programming (DP) matrix requirements.
- *  Use the macros:
  *
- *  - #VRNA_OPTION_MFE
- *  - #VRNA_OPTION_PF
- *  - #VRNA_OPTION_EVAL_ONLY
- *  - #VRNA_OPTION_DEFAULT
+ *  #### Options ####
+ *  * #VRNA_OPTION_DEFAULT  - @copybrief #VRNA_OPTION_DEFAULT
+ *  * #VRNA_OPTION_MFE      - @copybrief #VRNA_OPTION_MFE
+ *  * #VRNA_OPTION_PF       - @copybrief #VRNA_OPTION_PF
+ *  * #VRNA_OPTION_WINDOW   - @copybrief #VRNA_OPTION_WINDOW
  *
- *  to specify the required type of computations that will be performed with the #vrna_fold_compound_t.
+ *  The above options may be OR-ed together.
  *
  *  If you just need the folding compound serving as a container for your data, you can simply pass
  *  #VRNA_OPTION_DEFAULT to the @p option parameter. This creates a #vrna_fold_compound_t without DP
@@ -453,7 +460,7 @@ vrna_fold_compound(const char   *sequence,
  *  @param    sequences   A sequence alignment including 'gap' characters
  *  @param    md_p        An optional set of model details
  *  @param    options     The options for DP matrices memory allocation
- *  @return               A prefilled vrna_fold_compound_t that can be readily used for computations
+ *  @return               A prefilled vrna_fold_compound_t ready to be used for computations (may be @p NULL on error)
  */
 vrna_fold_compound_t *
 vrna_fold_compound_comparative(const char   **sequences,
@@ -461,6 +468,16 @@ vrna_fold_compound_comparative(const char   **sequences,
                                unsigned int options);
 
 
+vrna_fold_compound_t *
+vrna_fold_compound_comparative2(const char                **sequences,
+                                const char                **names,
+                                const unsigned char       *orientation,
+                                const unsigned long long  *start,
+                                const unsigned long long  *genome_size,
+                                vrna_md_t                 *md_p,
+                                unsigned int              options);
+
+
 vrna_fold_compound_t *
 vrna_fold_compound_TwoD(const char    *sequence,
                         const char    *s1,
@@ -470,7 +487,7 @@ vrna_fold_compound_TwoD(const char    *sequence,
 
 
 int
-vrna_fold_compound_prepare(vrna_fold_compound_t *vc,
+vrna_fold_compound_prepare(vrna_fold_compound_t *fc,
                            unsigned int         options);
 
 
@@ -479,10 +496,10 @@ vrna_fold_compound_prepare(vrna_fold_compound_t *vc,
  *
  *  @see vrna_fold_compound(), vrna_fold_compound_comparative(), vrna_mx_mfe_free(), vrna_mx_pf_free()
  *
- *  @param  vc  The #vrna_fold_compound_t that is to be erased from memory
+ *  @param  fc  The #vrna_fold_compound_t that is to be erased from memory
  */
 void
-vrna_fold_compound_free(vrna_fold_compound_t *vc);
+vrna_fold_compound_free(vrna_fold_compound_t *fc);
 
 
 /**
@@ -498,13 +515,14 @@ vrna_fold_compound_free(vrna_fold_compound_t *vc);
  *        on any pre-existing data that is already attached.
  *
  *  @see vrna_callback_free_auxdata()
- *  @param  vc    The fold_compound the arbitrary data pointer should be associated with
+ *  @param  fc    The fold_compound the arbitrary data pointer should be associated with
  *  @param  data  A pointer to an arbitrary data structure
  *  @param  f     A pointer to function that free's memory occupied by the arbitrary data (May be NULL)
  */
-void vrna_fold_compound_add_auxdata(vrna_fold_compound_t        *vc,
-                                    void                        *data,
-                                    vrna_callback_free_auxdata  *f);
+void
+vrna_fold_compound_add_auxdata(vrna_fold_compound_t       *fc,
+                               void                       *data,
+                               vrna_callback_free_auxdata *f);
 
 
 /**
@@ -519,11 +537,12 @@ void vrna_fold_compound_add_auxdata(vrna_fold_compound_t        *vc,
  *  @see  vrna_callback_recursion_status(), #vrna_fold_compound_t,
  *        #VRNA_STATUS_MFE_PRE, #VRNA_STATUS_MFE_POST, #VRNA_STATUS_PF_PRE, #VRNA_STATUS_PF_POST
  *
- *  @param  vc    The fold_compound the callback function should be attached to
+ *  @param  fc    The fold_compound the callback function should be attached to
  *  @param  f     The pointer to the recursion status callback function
  */
-void vrna_fold_compound_add_callback(vrna_fold_compound_t           *vc,
-                                     vrna_callback_recursion_status *f);
+void
+vrna_fold_compound_add_callback(vrna_fold_compound_t            *fc,
+                                vrna_callback_recursion_status  *f);
 
 
 /**
