@@ -1,8 +1,9 @@
 #ifndef VIENNA_RNA_PACKAGE_GQUAD_H
 #define VIENNA_RNA_PACKAGE_GQUAD_H
 
-#include <ViennaRNA/data_structures.h>
-#include <ViennaRNA/params.h>
+#include <ViennaRNA/datastructures/basic.h>
+#include <ViennaRNA/fold_compound.h>
+#include <ViennaRNA/params/basic.h>
 
 #ifndef INLINE
 #ifdef __GNUC__
@@ -26,29 +27,25 @@
  */
 
 
-int         E_gquad(int L,
-                    int l[3],
-                    vrna_param_t * P);
-
-FLT_OR_DBL  exp_E_gquad(int L,
-                        int l[3],
-                        vrna_exp_param_t * pf);
-
-int         E_gquad_ali(int i,
-                        int L,
-                        int l[3],
-                        const short **S,
-                        int n_seq,
-                        vrna_param_t * P);
+int         E_gquad(int           L,
+                    int           l[3],
+                    vrna_param_t  *P);
 
 
-void E_gquad_ali_en(int i,
-                    int L,
-                    int l[3],
-                    const short **S,
-                    int n_seq,
-                    int en[2],
-                    vrna_param_t * P);
+FLT_OR_DBL  exp_E_gquad(int               L,
+                        int               l[3],
+                        vrna_exp_param_t  *pf);
+
+
+void E_gquad_ali_en(int           i,
+                    int           L,
+                    int           l[3],
+                    const short   **S,
+                    unsigned int  **a2s,
+                    unsigned int  n_seq,
+                    vrna_param_t  *P,
+                    int           en[2]);
+
 
 /**
  *  @brief Get a triangular matrix prefilled with minimum free energy
@@ -72,6 +69,7 @@ int *get_gquad_matrix(short         *S,
 
 int *get_gquad_ali_matrix(short         *S_cons,
                           short         **S,
+                          unsigned int  **a2s,
                           int           n_seq,
                           vrna_param_t  *P);
 
@@ -81,6 +79,14 @@ FLT_OR_DBL *get_gquad_pf_matrix(short             *S,
                                 vrna_exp_param_t  *pf);
 
 
+FLT_OR_DBL *get_gquad_pf_matrix_comparative(short             *S_cons,
+                                            short             **S,
+                                            unsigned int      **a2s,
+                                            FLT_OR_DBL        *scale,
+                                            unsigned int      n_seq,
+                                            vrna_exp_param_t  *pf);
+
+
 int **get_gquad_L_matrix(short        *S,
                          int          start,
                          int          maxdist,
@@ -93,12 +99,13 @@ void        vrna_gquad_mx_local_update(vrna_fold_compound_t *vc,
                                        int                  start);
 
 
-void get_gquad_pattern_mfe(short *S,
-                           int i,
-                           int j,
-                           vrna_param_t * P,
-                           int *L,
-                           int l[3]);
+void get_gquad_pattern_mfe(short        *S,
+                           int          i,
+                           int          j,
+                           vrna_param_t *P,
+                           int          *L,
+                           int          l[3]);
+
 
 void
 get_gquad_pattern_exhaustive(short        *S,
@@ -110,12 +117,13 @@ get_gquad_pattern_exhaustive(short        *S,
                              int          threshold);
 
 
-void get_gquad_pattern_pf(short *S,
-                          int i,
-                          int j,
-                          vrna_exp_param_t * pf,
-                          int *L,
-                          int l[3]);
+void get_gquad_pattern_pf(short             *S,
+                          int               i,
+                          int               j,
+                          vrna_exp_param_t  *pf,
+                          int               *L,
+                          int               l[3]);
+
 
 plist *get_plist_gquad_from_pr(short            *S,
                                int              gi,
@@ -126,20 +134,35 @@ plist *get_plist_gquad_from_pr(short            *S,
                                vrna_exp_param_t *pf);
 
 
-plist *get_plist_gquad_from_pr_max(short *S,
-                                   int gi,
-                                   int gj,
-                                   FLT_OR_DBL * G,
-                                   FLT_OR_DBL * probs,
-                                   FLT_OR_DBL * scale,
-                                   int *L,
-                                   int l[3],
-                                   vrna_exp_param_t * pf);
+plist *get_plist_gquad_from_pr_max(short            *S,
+                                   int              gi,
+                                   int              gj,
+                                   FLT_OR_DBL       *G,
+                                   FLT_OR_DBL       *probs,
+                                   FLT_OR_DBL       *scale,
+                                   int              *L,
+                                   int              l[3],
+                                   vrna_exp_param_t *pf);
+
 
 plist *get_plist_gquad_from_db(const char *structure,
                                float      pr);
 
 
+plist *
+vrna_get_plist_gquad_from_pr(vrna_fold_compound_t *fc,
+                             int                  gi,
+                             int                  gj);
+
+
+plist *
+vrna_get_plist_gquad_from_pr_max(vrna_fold_compound_t *fc,
+                                 int                  gi,
+                                 int                  gj,
+                                 int                  *Lmax,
+                                 int                  lmax[3]);
+
+
 int         get_gquad_count(short *S,
                             int   i,
                             int   j);
@@ -150,14 +173,16 @@ int         get_gquad_layer_count(short *S,
                                   int   j);
 
 
-void get_gquad_pattern_mfe_ali(short **S,
-                               short *S_cons,
-                               int n_seq,
-                               int i,
-                               int j,
-                               vrna_param_t * P,
-                               int *L,
-                               int l[3]);
+void get_gquad_pattern_mfe_ali(short        **S,
+                               unsigned int **a2s,
+                               short        *S_cons,
+                               int          n_seq,
+                               int          i,
+                               int          j,
+                               vrna_param_t *P,
+                               int          *L,
+                               int          l[3]);
+
 
 /**
  *  given a dot-bracket structure (possibly) containing gquads encoded
@@ -169,7 +194,10 @@ void get_gquad_pattern_mfe_ali(short **S,
  *  end2 = parse_gquad(struc+end1, &L, l); end2+=end1; ... ;
  *  end3 = parse_gquad(struc+end2, &L, l); end3+=end2; ... ;
  */
-int parse_gquad(const char *struc, int *L, int l[3]);
+int parse_gquad(const char  *struc,
+                int         *L,
+                int         l[3]);
+
 
 INLINE PRIVATE int backtrack_GQuad_IntLoop(int          c,
                                            int          i,
@@ -186,10 +214,11 @@ INLINE PRIVATE int backtrack_GQuad_IntLoop(int          c,
 INLINE PRIVATE int backtrack_GQuad_IntLoop_comparative(int          c,
                                                        int          i,
                                                        int          j,
-                                                       int          *type,
+                                                       unsigned int *type,
                                                        short        *S_cons,
                                                        short        **S5,
                                                        short        **S3,
+                                                       unsigned int **a2s,
                                                        int          *ggg,
                                                        int          *index,
                                                        int          *p,
@@ -230,7 +259,7 @@ vrna_BT_gquad_mfe(vrna_fold_compound_t  *vc,
    * here we do some fancy stuff to backtrace the stacksize and linker lengths
    * of the g-quadruplex that should reside within position i,j
    */
-  short         *S, *S_cons;
+  short         *S;
   int           l[3], L, a, n_seq;
   vrna_param_t  *P;
 
@@ -247,7 +276,7 @@ vrna_BT_gquad_mfe(vrna_fold_compound_t  *vc,
       case VRNA_FC_TYPE_COMPARATIVE:
         n_seq = vc->n_seq;
         L     = -1;
-        get_gquad_pattern_mfe_ali(vc->S, vc->S_cons, n_seq, i, j, P, &L, l);
+        get_gquad_pattern_mfe_ali(vc->S, vc->a2s, vc->S_cons, n_seq, i, j, P, &L, l);
         break;
     }
 
@@ -281,7 +310,7 @@ vrna_BT_gquad_int(vrna_fold_compound_t  *vc,
                   vrna_bp_stack_t       *bp_stack,
                   int                   *stack_count)
 {
-  int           energy, dangles, *idx, ij, p, q, maxl, minl, c0, l1, *rtype, *ggg;
+  int           energy, dangles, *idx, ij, p, q, maxl, minl, c0, l1, *ggg;
   unsigned char type;
   char          *ptype;
   short         si, sj, *S, *S1;
@@ -294,8 +323,7 @@ vrna_BT_gquad_int(vrna_fold_compound_t  *vc,
   P       = vc->params;
   md      = &(P->model_details);
   ptype   = vc->ptype;
-  rtype   = &(md->rtype[0]);
-  type    = rtype[(unsigned char)ptype[ij]];
+  type    = (unsigned char)ptype[ij];
   S1      = vc->sequence_encoding;
   S       = vc->sequence_encoding2;
   dangles = md->dangles;
@@ -494,10 +522,11 @@ INLINE PRIVATE int
 backtrack_GQuad_IntLoop_comparative(int           c,
                                     int           i,
                                     int           j,
-                                    int           *type,
+                                    unsigned int  *type,
                                     short         *S_cons,
                                     short         **S5,
                                     short         **S3,
+                                    unsigned int  **a2s,
                                     int           *ggg,
                                     int           *index,
                                     int           *p,
@@ -505,7 +534,7 @@ backtrack_GQuad_IntLoop_comparative(int           c,
                                     int           n_seq,
                                     vrna_param_t  *P)
 {
-  int energy, dangles, k, l, maxl, minl, c0, l1, ss, tt;
+  int energy, dangles, k, l, maxl, minl, c0, l1, ss, tt, u1, u2, eee;
 
   dangles = P->model_details.dangles;
   energy  = 0;
@@ -535,7 +564,14 @@ backtrack_GQuad_IntLoop_comparative(int           c,
         if (S_cons[l] != 3)
           continue;
 
-        if (c == energy + ggg[index[l] + k] + n_seq * P->internal_loop[j - l - 1]) {
+        eee = 0;
+
+        for (ss = 0; ss < n_seq; ss++) {
+          u1  = a2s[ss][j - 1] - a2s[ss][l];
+          eee += P->internal_loop[u1];
+        }
+
+        if (c == energy + ggg[index[l] + k] + eee) {
           *p  = k;
           *q  = l;
           return 1;
@@ -564,7 +600,15 @@ backtrack_GQuad_IntLoop_comparative(int           c,
       if (S_cons[l] != 3)
         continue;
 
-      if (c == energy + ggg[index[l] + k] + n_seq * P->internal_loop[l1 + j - l - 1]) {
+      eee = 0;
+
+      for (ss = 0; ss < n_seq; ss++) {
+        u1  = a2s[ss][k - 1] - a2s[ss][i];
+        u2  = a2s[ss][j - 1] - a2s[ss][l];
+        eee += P->internal_loop[u1 + u2];
+      }
+
+      if (c == energy + ggg[index[l] + k] + eee) {
         *p  = k;
         *q  = l;
         return 1;
@@ -584,7 +628,14 @@ backtrack_GQuad_IntLoop_comparative(int           c,
       if (S_cons[k] != 3)
         continue;
 
-      if (c == energy + ggg[index[l] + k] + n_seq * P->internal_loop[l1]) {
+      eee = 0;
+
+      for (ss = 0; ss < n_seq; ss++) {
+        u1  = a2s[ss][k - 1] - a2s[ss][i];
+        eee += P->internal_loop[u1];
+      }
+
+      if (c == energy + ggg[index[l] + k] + eee) {
         *p  = k;
         *q  = l;
         return 1;
@@ -714,10 +765,11 @@ INLINE PRIVATE int
 backtrack_GQuad_IntLoop_L_comparative(int           c,
                                       int           i,
                                       int           j,
-                                      int           *type,
+                                      unsigned int  *type,
                                       short         *S_cons,
                                       short         **S5,
                                       short         **S3,
+                                      unsigned int  **a2s,
                                       int           **ggg,
                                       int           *p,
                                       int           *q,
@@ -730,7 +782,7 @@ backtrack_GQuad_IntLoop_L_comparative(int           c,
    * kind and the enclosed pair is not a canonical one but a g-quadruplex
    * that should then be decomposed further...
    */
-  int mm, dangle_model, k, l, maxl, minl, c0, l1, ss, tt;
+  int mm, dangle_model, k, l, maxl, minl, c0, l1, ss, tt, eee, u1, u2;
 
   dangle_model = P->model_details.dangles;
 
@@ -765,7 +817,18 @@ backtrack_GQuad_IntLoop_L_comparative(int           c,
       if (S_cons[l] != 3)
         continue;
 
-      c0 = mm + ggg[k][l - k] + n_seq * P->internal_loop[l1 + j - l - 1];
+      eee = 0;
+
+      for (ss = 0; ss < n_seq; ss++) {
+        u1  = a2s[ss][k - 1] - a2s[ss][i];
+        u2  = a2s[ss][j - 1] - a2s[ss][l];
+        eee += P->internal_loop[u1 + u2];
+      }
+
+      c0 = mm +
+           ggg[k][l - k] +
+           eee;
+
       if (c == c0) {
         *p  = k;
         *q  = l;
@@ -786,7 +849,14 @@ backtrack_GQuad_IntLoop_L_comparative(int           c,
         if (S_cons[l] != 3)
           continue;
 
-        if (c == mm + ggg[k][l - k] + n_seq * P->internal_loop[j - l - 1]) {
+        eee = 0;
+
+        for (ss = 0; ss < n_seq; ss++) {
+          u1  = a2s[ss][j - 1] - a2s[ss][l];
+          eee += P->internal_loop[u1];
+        }
+
+        if (c == mm + ggg[k][l - k] + eee) {
           *p  = k;
           *q  = l;
           return 1;
@@ -805,7 +875,14 @@ backtrack_GQuad_IntLoop_L_comparative(int           c,
       if (S_cons[k] != 3)
         continue;
 
-      if (c == mm + ggg[k][l - k] + n_seq * P->internal_loop[l1]) {
+      eee = 0;
+
+      for (ss = 0; ss < n_seq; ss++) {
+        u1  = a2s[ss][k - 1] - a2s[ss][i];
+        eee += P->internal_loop[u1];
+      }
+
+      if (c == mm + ggg[k][l - k] + eee) {
         *p  = k;
         *q  = l;
         return 1;
@@ -975,6 +1052,253 @@ E_GQuad_IntLoop(int           i,
 }
 
 
+PRIVATE INLINE
+int
+E_GQuad_IntLoop_comparative(int           i,
+                            int           j,
+                            unsigned int  *tt,
+                            short         *S_cons,
+                            short         **S5,
+                            short         **S3,
+                            unsigned int  **a2s,
+                            int           *ggg,
+                            int           *index,
+                            int           n_seq,
+                            vrna_param_t  *P)
+{
+  unsigned int  type;
+  int           eee, energy, ge, p, q, l1, u1, u2, minq, maxq, c0, s;
+  vrna_md_t     *md;
+
+  md      = &(P->model_details);
+  energy  = 0;
+
+  for (s = 0; s < n_seq; s++) {
+    type = tt[s];
+    if (md->dangles == 2)
+      energy += P->mismatchI[type][S3[s][i]][S5[s][j]];
+
+    if (type > 2)
+      energy += P->TerminalAU;
+  }
+
+  ge = INF;
+
+  p = i + 1;
+  if (S_cons[p] == 3) {
+    if (p < j - VRNA_GQUAD_MIN_BOX_SIZE) {
+      minq  = j - i + p - MAXLOOP - 2;
+      c0    = p + VRNA_GQUAD_MIN_BOX_SIZE - 1;
+      minq  = MAX2(c0, minq);
+      c0    = j - 3;
+      maxq  = p + VRNA_GQUAD_MAX_BOX_SIZE + 1;
+      maxq  = MIN2(c0, maxq);
+      for (q = minq; q < maxq; q++) {
+        if (S_cons[q] != 3)
+          continue;
+
+        eee = 0;
+
+        for (s = 0; s < n_seq; s++) {
+          u1  = a2s[s][j - 1] - a2s[s][q];
+          eee += P->internal_loop[u1];
+        }
+
+        c0 = energy +
+             ggg[index[q] + p] +
+             eee;
+        ge = MIN2(ge, c0);
+      }
+    }
+  }
+
+  for (p = i + 2;
+       p < j - VRNA_GQUAD_MIN_BOX_SIZE;
+       p++) {
+    l1 = p - i - 1;
+    if (l1 > MAXLOOP)
+      break;
+
+    if (S_cons[p] != 3)
+      continue;
+
+    minq  = j - i + p - MAXLOOP - 2;
+    c0    = p + VRNA_GQUAD_MIN_BOX_SIZE - 1;
+    minq  = MAX2(c0, minq);
+    c0    = j - 1;
+    maxq  = p + VRNA_GQUAD_MAX_BOX_SIZE + 1;
+    maxq  = MIN2(c0, maxq);
+    for (q = minq; q < maxq; q++) {
+      if (S_cons[q] != 3)
+        continue;
+
+      eee = 0;
+
+      for (s = 0; s < n_seq; s++) {
+        u1  = a2s[s][p - 1] - a2s[s][i];
+        u2  = a2s[s][j - 1] - a2s[s][q];
+        eee += P->internal_loop[u1 + u2];
+      }
+
+      c0 = energy +
+           ggg[index[q] + p] +
+           eee;
+      ge = MIN2(ge, c0);
+    }
+  }
+
+  q = j - 1;
+  if (S_cons[q] == 3)
+    for (p = i + 4;
+         p < j - VRNA_GQUAD_MIN_BOX_SIZE;
+         p++) {
+      l1 = p - i - 1;
+      if (l1 > MAXLOOP)
+        break;
+
+      if (S_cons[p] != 3)
+        continue;
+
+      eee = 0;
+
+      for (s = 0; s < n_seq; s++) {
+        u1  = a2s[s][p - 1] - a2s[s][i];
+        eee += P->internal_loop[u1];
+      }
+
+      c0 = energy +
+           ggg[index[q] + p] +
+           eee;
+      ge = MIN2(ge, c0);
+    }
+
+  return ge;
+}
+
+
+PRIVATE INLINE
+int
+E_GQuad_IntLoop_L_comparative(int           i,
+                              int           j,
+                              unsigned int  *tt,
+                              short         *S_cons,
+                              short         **S5,
+                              short         **S3,
+                              unsigned int  **a2s,
+                              int           **ggg,
+                              int           n_seq,
+                              vrna_param_t  *P)
+{
+  unsigned int  type;
+  int           eee, energy, ge, p, q, l1, u1, u2, minq, maxq, c0, s;
+  vrna_md_t     *md;
+
+  md      = &(P->model_details);
+  energy  = 0;
+
+  for (s = 0; s < n_seq; s++) {
+    type = tt[s];
+    if (md->dangles == 2)
+      energy += P->mismatchI[type][S3[s][i]][S5[s][j]];
+
+    if (type > 2)
+      energy += P->TerminalAU;
+  }
+
+  ge = INF;
+
+  p = i + 1;
+  if (S_cons[p] == 3) {
+    if (p < j - VRNA_GQUAD_MIN_BOX_SIZE) {
+      minq  = j - i + p - MAXLOOP - 2;
+      c0    = p + VRNA_GQUAD_MIN_BOX_SIZE - 1;
+      minq  = MAX2(c0, minq);
+      c0    = j - 3;
+      maxq  = p + VRNA_GQUAD_MAX_BOX_SIZE + 1;
+      maxq  = MIN2(c0, maxq);
+      for (q = minq; q < maxq; q++) {
+        if (S_cons[q] != 3)
+          continue;
+
+        eee = 0;
+
+        for (s = 0; s < n_seq; s++) {
+          u1  = a2s[s][j - 1] - a2s[s][q];
+          eee += P->internal_loop[u1];
+        }
+
+        c0 = energy +
+             ggg[p][q - p] +
+             eee;
+        ge = MIN2(ge, c0);
+      }
+    }
+  }
+
+  for (p = i + 2;
+       p < j - VRNA_GQUAD_MIN_BOX_SIZE;
+       p++) {
+    l1 = p - i - 1;
+    if (l1 > MAXLOOP)
+      break;
+
+    if (S_cons[p] != 3)
+      continue;
+
+    minq  = j - i + p - MAXLOOP - 2;
+    c0    = p + VRNA_GQUAD_MIN_BOX_SIZE - 1;
+    minq  = MAX2(c0, minq);
+    c0    = j - 1;
+    maxq  = p + VRNA_GQUAD_MAX_BOX_SIZE + 1;
+    maxq  = MIN2(c0, maxq);
+    for (q = minq; q < maxq; q++) {
+      if (S_cons[q] != 3)
+        continue;
+
+      eee = 0;
+
+      for (s = 0; s < n_seq; s++) {
+        u1  = a2s[s][p - 1] - a2s[s][i];
+        u2  = a2s[s][j - 1] - a2s[s][q];
+        eee += P->internal_loop[u1 + u2];
+      }
+
+      c0 = energy +
+           ggg[p][q - p] +
+           eee;
+      ge = MIN2(ge, c0);
+    }
+  }
+
+  q = j - 1;
+  if (S_cons[q] == 3)
+    for (p = i + 4;
+         p < j - VRNA_GQUAD_MIN_BOX_SIZE;
+         p++) {
+      l1 = p - i - 1;
+      if (l1 > MAXLOOP)
+        break;
+
+      if (S_cons[p] != 3)
+        continue;
+
+      eee = 0;
+
+      for (s = 0; s < n_seq; s++) {
+        u1  = a2s[s][p - 1] - a2s[s][i];
+        eee += P->internal_loop[u1];
+      }
+
+      c0 = energy +
+           ggg[p][q - p] +
+           eee;
+      ge = MIN2(ge, c0);
+    }
+
+  return ge;
+}
+
+
 PRIVATE INLINE
 int *
 E_GQuad_IntLoop_exhaustive(int          i,
@@ -1278,6 +1602,140 @@ exp_E_GQuad_IntLoop(int               i,
 }
 
 
+PRIVATE INLINE
+FLT_OR_DBL
+exp_E_GQuad_IntLoop_comparative(int               i,
+                                int               j,
+                                unsigned int      *tt,
+                                short             *S_cons,
+                                short             **S5,
+                                short             **S3,
+                                unsigned int      **a2s,
+                                FLT_OR_DBL        *G,
+                                FLT_OR_DBL        *scale,
+                                int               *index,
+                                int               n_seq,
+                                vrna_exp_param_t  *pf)
+{
+  unsigned int  type;
+  int           k, l, minl, maxl, u, u1, u2, r, s;
+  FLT_OR_DBL    q, qe, qqq;
+  double        *expintern;
+  vrna_md_t     *md;
+
+  q         = 0;
+  qe        = 1.;
+  md        = &(pf->model_details);
+  expintern = &(pf->expinternal[0]);
+
+  for (s = 0; s < n_seq; s++) {
+    type = tt[s];
+    if (md->dangles == 2)
+      qe *= (FLT_OR_DBL)pf->expmismatchI[type][S3[s][i]][S5[s][j]];
+
+    if (type > 2)
+      qe *= (FLT_OR_DBL)pf->expTermAU;
+  }
+
+  k = i + 1;
+  if (S_cons[k] == 3) {
+    if (k < j - VRNA_GQUAD_MIN_BOX_SIZE) {
+      minl  = j - MAXLOOP - 1;
+      u     = k + VRNA_GQUAD_MIN_BOX_SIZE - 1;
+      minl  = MAX2(u, minl);
+      u     = j - 3;
+      maxl  = k + VRNA_GQUAD_MAX_BOX_SIZE + 1;
+      maxl  = MIN2(u, maxl);
+      for (l = minl; l < maxl; l++) {
+        if (S_cons[l] != 3)
+          continue;
+
+        if (G[index[k] - l] == 0.)
+          continue;
+
+        qqq = 1.;
+
+        for (s = 0; s < n_seq; s++) {
+          u1  = a2s[s][j - 1] - a2s[s][l];
+          qqq *= expintern[u1];
+        }
+
+        q += qe *
+             G[index[k] - l] *
+             qqq *
+             scale[j - l + 1];
+      }
+    }
+  }
+
+  for (k = i + 2;
+       k <= j - VRNA_GQUAD_MIN_BOX_SIZE;
+       k++) {
+    u = k - i - 1;
+    if (u > MAXLOOP)
+      break;
+
+    if (S_cons[k] != 3)
+      continue;
+
+    minl  = j - i + k - MAXLOOP - 2;
+    r     = k + VRNA_GQUAD_MIN_BOX_SIZE - 1;
+    minl  = MAX2(r, minl);
+    maxl  = k + VRNA_GQUAD_MAX_BOX_SIZE + 1;
+    r     = j - 1;
+    maxl  = MIN2(r, maxl);
+    for (l = minl; l < maxl; l++) {
+      if (S_cons[l] != 3)
+        continue;
+
+      if (G[index[k] - l] == 0.)
+        continue;
+
+      qqq = 1.;
+
+      for (s = 0; s < n_seq; s++) {
+        u1  = a2s[s][k - 1] - a2s[s][i];
+        u2  = a2s[s][j - 1] - a2s[s][l];
+        qqq *= expintern[u1 + u2];
+      }
+
+      q += qe *
+           G[index[k] - l] *
+           qqq *
+           scale[u + j - l + 1];
+    }
+  }
+
+  l = j - 1;
+  if (S_cons[l] == 3)
+    for (k = i + 4; k <= j - VRNA_GQUAD_MIN_BOX_SIZE; k++) {
+      u = k - i - 1;
+      if (u > MAXLOOP)
+        break;
+
+      if (S_cons[k] != 3)
+        continue;
+
+      if (G[index[k] - l] == 0.)
+        continue;
+
+      qqq = 1.;
+
+      for (s = 0; s < n_seq; s++) {
+        u1  = a2s[s][k - 1] - a2s[s][i];
+        qqq *= expintern[u1];
+      }
+
+      q += qe *
+           G[index[k] - l] *
+           qqq *
+           scale[u + 2];
+    }
+
+  return q;
+}
+
+
 /**
  * @}
  */
