@@ -29,6 +29,7 @@
 #include "ViennaRNA/ribo.h"
 #include "ViennaRNA/utils/alignments.h"
 #include "ViennaRNA/alphabet.h"
+#include "ViennaRNA/constraints/hard.h"
 #include "ViennaRNA/mfe_window.h"
 
 
@@ -515,7 +516,7 @@ init_constraints(vrna_fold_compound_t *fc,
     case VRNA_FC_TYPE_SINGLE:
       for (i = length; (i >= length - maxdist - 4) && (i > 0); i--) {
         make_ptypes(fc, i);
-        vrna_hc_update(fc, i);
+        vrna_hc_update(fc, i, VRNA_CONSTRAINT_WINDOW_UPDATE_3);
         vrna_sc_update(fc, i, VRNA_OPTION_MFE | VRNA_OPTION_WINDOW);
       }
       break;
@@ -523,7 +524,7 @@ init_constraints(vrna_fold_compound_t *fc,
     case VRNA_FC_TYPE_COMPARATIVE:
       for (i = length; (i >= length - maxdist - 4) && (i > 0); i--) {
         make_pscores(fc, i, dm);
-        vrna_hc_update(fc, i);
+        vrna_hc_update(fc, i, VRNA_CONSTRAINT_WINDOW_UPDATE_3);
       }
 
       /* for noLP option */
@@ -552,7 +553,7 @@ rotate_constraints(vrna_fold_compound_t *fc,
         fc->ptype_local[i + maxdist + 4]  = NULL;
         if (i > 1) {
           make_ptypes(fc, i - 1);
-          vrna_hc_update(fc, i - 1);
+          vrna_hc_update(fc, i - 1, VRNA_CONSTRAINT_WINDOW_UPDATE_3);
           vrna_sc_update(fc, i - 1, VRNA_OPTION_MFE | VRNA_OPTION_WINDOW);
         }
       }
@@ -567,7 +568,7 @@ rotate_constraints(vrna_fold_compound_t *fc,
           if (i > 2)
             make_pscores(fc, i - 2, dm);
 
-          vrna_hc_update(fc, i - 1);
+          vrna_hc_update(fc, i - 1, VRNA_CONSTRAINT_WINDOW_UPDATE_3);
         } else if (i == 1) {
           free(fc->pscore_local[i - 1]);
           fc->pscore_local[i - 1]           = fc->pscore_local[i + maxdist + 4];
