@@ -19,19 +19,20 @@
 #include "ViennaRNA/fold.h"
 #include "ViennaRNA/part_func.h"
 #include "ViennaRNA/fold_vars.h"
-#include "ViennaRNA/plot_aln.h"
-#include "ViennaRNA/plot_structure.h"
-#include "ViennaRNA/plot_utils.h"
-#include "ViennaRNA/utils.h"
+#include "ViennaRNA/plotting/alignments.h"
+#include "ViennaRNA/plotting/structures.h"
+#include "ViennaRNA/plotting/utils.h"
+#include "ViennaRNA/utils/basic.h"
+#include "ViennaRNA/utils/strings.h"
 #include "ViennaRNA/alifold.h"
 #include "ViennaRNA/Lfold.h"
-#include "ViennaRNA/aln_util.h"
-#include "ViennaRNA/read_epars.h"
+#include "ViennaRNA/utils/alignments.h"
+#include "ViennaRNA/params/io.h"
 #include "ViennaRNA/alphabet.h"
-#include "ViennaRNA/file_formats_msa.h"
-#include "ViennaRNA/file_utils.h"
-#include "ViennaRNA/constraints.h"
-#include "ViennaRNA/constraints_SHAPE.h"
+#include "ViennaRNA/io/file_formats_msa.h"
+#include "ViennaRNA/io/utils.h"
+#include "ViennaRNA/constraints/basic.h"
+#include "ViennaRNA/constraints/SHAPE.h"
 #include "RNALalifold_cmdl.h"
 #include "gengetopt_helper.h"
 #include "input_id_helpers.h"
@@ -75,18 +76,18 @@ main(int  argc,
   FILE                          *clust_file;
   struct RNALalifold_args_info  args_info;
   char                          *string, *structure, *prefix, *tmp_string,
-                                **AS, **names, *filename_in, *id_prefix, *id_delim, *filename_delim,
-                                **input_files, *tmp_id, *tmp_structure, **shape_files,
-                                *shape_method;
+                                **AS, **names, *filename_in, *filename_delim, **input_files,
+                                *tmp_id, *tmp_structure, **shape_files, *shape_method;
   unsigned int                  input_format_options, longest_string, aln_options;
-  int                           n_seq, i, maxdist, unchangednc, unchangedcv, quiet, auto_id,
-                                mis, istty, alnPS, aln_columns, aln_out, ssPS, input_file_num,
-                                id_digits, with_shapes, *shape_file_association, verbose, s,
-                                tmp_number, split_contributions;
-  long int                      alignment_number, first_alignment_number;
+  int                           n_seq, i, maxdist, unchangednc, unchangedcv, quiet, mis, istty,
+                                alnPS, aln_columns, aln_out, ssPS, input_file_num, with_shapes,
+                                *shape_file_association, verbose, s, tmp_number,
+                                split_contributions;
+  long int                      first_alignment_number;
   float                         e_max;
   vrna_md_t                     md;
   vrna_fold_compound_t          *fc;
+  dataset_id                    id_control;
 
   clust_file              = stdin;
   string                  = structure = prefix = NULL;
@@ -109,7 +110,6 @@ main(int  argc,
   with_shapes             = 0;
   verbose                 = 0;
   quiet                   = 0;
-  auto_id                 = 0;
   e_max                   = -0.1; /* threshold in kcal/mol per nucleotide in a hit */
   split_contributions     = 0;
 
@@ -127,28 +127,26 @@ main(int  argc,
   ggo_get_md_eval(args_info, md);
   ggo_get_md_fold(args_info, md);
 
+  /* temperature */
+  ggo_get_temperature(args_info, md.temperature);
+
   /* check dangle model */
   if ((md.dangles < 0) || (md.dangles > 3)) {
     vrna_message_warning("required dangle model not implemented, falling back to default dangles=2");
     md.dangles = 2;
   }
 
-  ggo_get_ID_manipulation(args_info,
-                          auto_id,
-                          id_prefix, "alignment",
-                          id_delim, "_",
-                          id_digits, 4,
-                          alignment_number, 1);
+  ggo_get_id_control(args_info, id_control, "Alignment", "alignment", "_", 4, 1);
 
   /* set cfactor */
   if (args_info.cfactor_given) {
-    md.cv_fact  = cv_fact = args_info.cfactor_arg;
+    md.cv_fact  = args_info.cfactor_arg;
     unchangedcv = 0;
   }
 
   /* set nfactor */
   if (args_info.nfactor_given) {
-    md.nc_fact  = nc_fact = args_info.nfactor_arg;
+    md.nc_fact  = args_info.nfactor_arg;
     unchangednc = 0;
   }
 
@@ -304,10 +302,12 @@ main(int  argc,
   /* filename sanitize delimiter */
   if (args_info.filename_delim_given)
     filename_delim = strdup(args_info.filename_delim_arg);
+  else if (get_id_delim(id_control))
+    filename_delim = strdup(get_id_delim(id_control));
   else
-    filename_delim = strdup(id_delim);
+    filename_delim = NULL;
 
-  if (isspace(*filename_delim)) {
+  if ((filename_delim) && isspace(*filename_delim)) {
     free(filename_delim);
     filename_delim = NULL;
   }
@@ -331,10 +331,10 @@ main(int  argc,
    #############################################
    */
   if ((ribo == 1) && (unchangednc))
-    md.nc_fact = nc_fact = 0.5;
+    md.nc_fact = 0.5;
 
   if ((ribo == 1) && (unchangedcv))
-    md.cv_fact = cv_fact = 0.6;
+    md.cv_fact = 0.6;
 
   istty = isatty(fileno(stdout)) && isatty(fileno(stdin));
 
@@ -379,32 +379,32 @@ main(int  argc,
     input_format_options = format_guess;
   }
 
-  first_alignment_number = alignment_number;
+  first_alignment_number = get_current_id(id_control);
 
   while (!feof(clust_file)) {
     char  **MSA_orig  = NULL;
     char  *MSA_ID     = NULL;
     fflush(stdout);
     if (istty && (clust_file == stdin)) {
-      switch (input_format_options) {
+      switch (input_format_options & (~VRNA_FILE_FORMAT_MSA_QUIET)) {
         case VRNA_FILE_FORMAT_MSA_CLUSTAL:
-          vrna_message_input_seq("Input aligned sequences in ClustalW format\n"
-                                 "press Ctrl+d when finished to indicate the end of your input)");
+          vrna_message_input_msa("Input aligned sequences in ClustalW format\n"
+                                 "(press Ctrl+d when finished to indicate the end of your input)");
           break;
 
         case VRNA_FILE_FORMAT_MSA_STOCKHOLM:
-          vrna_message_input_seq("Input aligned sequences in Stockholm format (Insert one alignment at a time!)\n"
-                                 "press Ctrl+d when finished to indicate the end of your input)");
+          vrna_message_input_msa("Input aligned sequences in Stockholm format (Insert one alignment at a time!)\n"
+                                 "(Note, Stockholm entries always end with a line that only contains '//'");
           break;
 
         case VRNA_FILE_FORMAT_MSA_FASTA:
-          vrna_message_input_seq("Input aligned sequences in FASTA format\n"
-                                 "press Ctrl+d when finished to indicate the end of your input)");
+          vrna_message_input_msa("Input aligned sequences in FASTA format\n"
+                                 "(press Ctrl+d when finished to indicate the end of your input)");
           break;
 
         case VRNA_FILE_FORMAT_MSA_MAF:
-          vrna_message_input_seq("Input aligned sequences in MAF format (Insert one alignment at a time!)\n"
-                                 "press Ctrl+d when finished to indicate the end of your input)");
+          vrna_message_input_msa("Input aligned sequences in MAF format (Insert one alignment at a time!)\n"
+                                 "(Note, a MAF alignment always ends with an empty line)");
           break;
 
         default:
@@ -440,10 +440,7 @@ main(int  argc,
     }
 
     /* construct alignment ID */
-    if (tmp_id && (!auto_id)) /* we've read an ID from file, so we use it */
-      MSA_ID = strdup(tmp_id);
-    else if (auto_id)         /* we have nuffin', Jon Snow (...so we simply generate an ID) */
-      MSA_ID = vrna_strdup_printf("%s%s%0*ld", id_prefix, id_delim, id_digits, alignment_number);
+    MSA_ID = fileprefix_from_id_alifold(tmp_id, id_control, 0);
 
     print_fasta_header(stdout, MSA_ID);
 
@@ -499,7 +496,9 @@ main(int  argc,
 
     (void)vrna_mfe_window_cb(fc, &print_hit_cb, (void *)&data);
 
-    string = (mis) ? consens_mis((const char **)AS) : consensus((const char **)AS);
+    string =
+      (mis) ? vrna_aln_consensus_mis((const char **)AS,
+                                     &md) : vrna_aln_consensus_sequence((const char **)AS, &md);
     printf("%s\n", string);
 
     (void)fflush(stdout);
@@ -515,14 +514,12 @@ main(int  argc,
     free(tmp_id);
     free(tmp_structure);
 
-    ID_number_increase(&alignment_number, "Alignment");
-
     /* break after first record if constraint folding and not explicitly instructed otherwise */
     if (with_shapes)
       break;
   } /* end of input */
 
-  if (first_alignment_number == alignment_number) {
+  if (first_alignment_number == get_current_id(id_control)) {
     char *format = NULL;
     switch (input_format_options) {
       case VRNA_FILE_FORMAT_MSA_CLUSTAL:
@@ -558,10 +555,10 @@ main(int  argc,
     free(input_files[i]);
   free(input_files);
 
-  free(id_prefix);
-  free(id_delim);
   free(filename_delim);
 
+  free_id_data(id_control);
+
   return EXIT_SUCCESS;
 }
 
@@ -611,8 +608,10 @@ print_hit_cb(int        start,
 
   if ((en / (float)(end - start + 1)) <= threshold) {
     sub   = vrna_aln_slice((const char **)strings, (unsigned int)start, (unsigned int)end);
-    cons  = (with_mis) ? consens_mis((const char **)sub) : consensus((const char **)sub);
-    A     = vrna_annotate_covar_struct((const char **)sub, ss, md);
+    cons  =
+      (with_mis) ? vrna_aln_consensus_mis((const char **)sub,
+                                          md) : vrna_aln_consensus_sequence((const char **)sub, md);
+    A = vrna_annotate_covar_db((const char **)sub, ss, md);
 
 
     if (split_energies) {
@@ -689,21 +688,17 @@ print_hit_cb(int        start,
         fname = vrna_strdup_printf("aln_%d_%d.eps", start, end);
 
       tmp_string = vrna_filename_sanitize(fname, "_");
-      free(fname);
-      fname = tmp_string;
-      char **sub_orig = vrna_aln_slice((const char **)strings_orig,
-                                       (unsigned int)start,
-                                       (unsigned int)end);
 
-      vrna_file_PS_aln_sub(fname,
-                           (const char **)sub_orig,
-                           (const char **)names,
-                           ss,
-                           start,
-                           -1,
-                           columns);
+      vrna_file_PS_aln_slice(tmp_string,
+                             (const char **)strings_orig,
+                             (const char **)names,
+                             ss - start + 1,
+                             start,
+                             end,
+                             0,
+                             columns);
       free(fname);
-      vrna_aln_free(sub_orig);
+      free(tmp_string);
     }
 
     if (with_stk) {
