@@ -21,15 +21,18 @@
 #include <float.h>
 #include "ViennaRNA/fold.h"
 #include "ViennaRNA/fold_vars.h"
-#include "ViennaRNA/read_epars.h"
-#include "ViennaRNA/PS_dot.h"
-#include "ViennaRNA/utils.h"
+#include "ViennaRNA/params/io.h"
+#include "ViennaRNA/plotting/probabilities.h"
+#include "ViennaRNA/utils/basic.h"
+#include "ViennaRNA/utils/strings.h"
 #include "ViennaRNA/part_func.h"
 #include "ViennaRNA/part_func_up.h"
 #include "ViennaRNA/duplex.h"
-#include "ViennaRNA/energy_const.h"
-#include "ViennaRNA/file_formats.h"
-#include "ViennaRNA/constraints.h"
+#include "ViennaRNA/params/constants.h"
+#include "ViennaRNA/io/file_formats.h"
+#include "ViennaRNA/constraints/basic.h"
+#include "ViennaRNA/constraints/hard.h"
+#include "ViennaRNA/constraints/soft.h"
 #include "RNAup_cmdl.h"
 
 #define MIN(x, y) (((x) < (y)) ? (x) : (y))
@@ -90,7 +93,7 @@ main(int  argc,
                           *orig_target;
   int                     i, j, length1, length2, length_target, sym, istty,
                           rotated, noconv, max_u, **unpaired_values, ulength_num;
-  double                  energy, min_en, sfact;
+  double                  min_en, sfact;
 
   /* variables for output */
   pu_contrib              *unstr_out, *unstr_short, *unstr_target, *contrib1, *contrib2;
@@ -354,8 +357,12 @@ main(int  argc,
    #############################################
    */
 
-  if (ParamFile != NULL)
-    read_parameter_file(ParamFile);
+  if (ParamFile != NULL) {
+    if (!strcmp(ParamFile, "DNA"))
+        vrna_params_load_DNA_Mathews2004();
+    else
+      vrna_params_load(ParamFile, VRNA_PARAMETER_FORMAT_DEFAULT);
+  }
 
   if (ns_bases != NULL) {
     nonstandards  = vrna_alloc(33);
@@ -701,7 +708,7 @@ main(int  argc,
     if (cstruc1 != NULL)
       strncpy(structure, cstruc1, length1 + 1);
 
-    energy    = pf_fold(s1, structure);
+    (void) pf_fold(s1, structure);
     unstr_out = pf_unstru(s1, wplus);
     free_pf_arrays();
 
@@ -755,7 +762,7 @@ main(int  argc,
           if (cstruc_target != NULL)
             strncpy(structure, cstruc_target, length_target + 1);
 
-          energy        = pf_fold(s_target, structure);
+          (void) pf_fold(s_target, structure);
           unstr_target  = pf_unstru(s_target, wplus);
           free_pf_arrays();                     /* for arrays for pf_fold(...) */
         }
